pro mreadfits_urls, fitsurls,index, data, outsize=outsize, nodata=nodata, $  
   server=server, verbose=verbose, gateway=gateway
;+
;   Name: mreadfits_urls
;
;   Purpose: read one/multiple FITS urls -> "index,data" via sockets
;
;   Input Parameters:
;      fitsurls - list of one or more fits file urls
;
;   Output Parameters:
;      index [,data] - ssw standards: index->FITS header as structure vector
;                                     data ->2D or 3D data
;
;   Keyword Parameters:
;      server - desired server (default derived from FITSURLs input)   
;      gateway - (switch) if server/fitsurls are via gateway 
;
;   Calling Examples:
;      mreadfits_urls,fitsurls, index [,server=server] [,/gateway] ; header only->index
;      mreadfits_urls,fitsurls, index, data                        ; headers+data    
;
;   Calling Sample: (sxi multi image example )
;
;      IDL> sxiurls=sxi_files('1-dec-2001 02:20','1-dec-2001 02:50',/ngdc,/full) ; time->urls
;      IDL> mreadfits_urls,sxiurls,index, server=sxi_server(),/GATEWAY           ; headers ->index
;
;      IDL> help,xiurls,index                                                    ; show what I got
;           SXIURLS         STRING    = Array[32]
;           INDEX           STRUCT    = -> MS_097097004001 Array[32]
;      IDL> more,get_infox(index,'date_obs,exptime,wavelnth,crpix1,crpix2')      ; index summaries   
;         2001-12-01T02:19:14.004      3.0005     OPEN    224.0391    260.5744   ; (get_infox.pro)
;         2001-12-01T02:20:00.865      0.0155     OPEN    222.9005    259.9301
;         2001-12-01T02:21:15.507      3.0005  P_MED_A    221.9841    260.8560
;         2001-12-01T02:22:02.352      0.0255  P_MED_A    221.6006    261.2311
;         (...etc)
;
;      ; filter on above index, and read 3D subset
;      IDL> ss=where(index.exptime gt 2 and index.wavelnth eq 'OPEN')       ; desired subset  
;      IDL> mreadfits_urls,sxiurls(ss),index,data,server=sxi_server(),/GATE ; now read data als
;                          ============
;      IDL> help, 
;     
;   Calls:
;      ssw standards, fitshead2struct etc..
;      DMZarro hfits__define object (socket read)
;
;   History:
;      14-Jan-2003 - S.L.Freeland  ; socket/url analog of 'mreadfits.pro'
;      21-Jan-2003 - S.L.Freeland header-only socket reads & copy->local options
;;
;   Restrictions:
;      At least for today, assumes all files same  naxis1&naxis2
;      OUTSIZE not yet implemented 
;      Assumes same server for a given call
;-

if not data_chk(fitsurls,/string) then begin 
   box_message,'Requires url list...'
   return
endif

case 1 of 
   data_chk(server,/string):                       ; user supplied
   else: begin 
      server=ssw_strsplit(fitsurls(0),'/',/head)   ; from 1st input element
   endcase
endcase
server=server(0)
nimg=n_elements(fitsurls)
nodata=keyword_set(nodata) or n_params() lt 3
verbose=keyword_set(verbose)
copy_local=keyword_set(copy_local) or data_chk(outdir,/string)
needdata=(1-nodata) or copy_local

if n_elements(outdir) eq 0 then outdir=get_temp_dir()

nii=-1                                                         ; current file subscript 

sread=(['http->hread,fitsurls(nii),header0',$                  ; header only socket read
        'http->read ,fitsurls(nii),data0,header0'])(needdata)  ; header+data

; sread=([sread,'sock_copy,fitsurls(nii),data0,header0,outdir=outdir'])(copy_local)

if not copy_local then begin 
   http=obj_new('hfits')
   http->open,server,gateway=gateway
endif

while not data_chk(index0,/struct) and nii lt nimg do begin 
   nii=nii+1
   estat=execute(sread)
   if n_elements(header0) gt 1 then index0=fitshead2struct(header0)
endwhile 

case 1 of 
   not data_chk(index0,/struct): begin 
      box_message,'No files read properly, returning...
   endcase
   nii eq (nimg-1): begin                          ; all done 
      estat=execute((['','data=temporary(data0)'])(needdata))
      index=temporary(index0)
   endcase
   else: begin                                     ; more to do  
      if needdata then begin  
         data=make_array(data_chk(data0,/nx),data_chk(data0,/nx), nimg-(nii),$
                         type=data_chk(data0,/type),/nozero)
         data(0,0,0)=temporary(data0)
      endif
      index=replicate(index0,nimg) 
      for nii=nii+1,nimg-1 do begin
         delvarx,data0,header0
         estat=execute(sread)
         if verbose then print,'Just read> ' + fitsurls(nii)
         estat=execute((['','data(0,0,nii)=temporary(data0)'])(needdata)) ; 2D->3D
         index(nii)=temporary( fitshead2struct(header0,index(0)))         ; index
      endfor
   endcase 
endcase

if not copy_local then obj_destroy,http
   
return
end




